#pragma once

#include "vamp-sdk/Plugin.h"
#include "vamp.h"
#include <vector>

namespace vamp 
{
  class PluginHostAdapter : public plugin
  {
    public:
      PluginHostAdapter(const VampPluginDescriptor* descriptor, float inputSampleRate);
      virtual ~PluginHostAdapter();
      
      static std::vector<std::string> getPluginPath();

      bool initialise(size_t channels, size_t stepSize, size_t blockSize) override;
      void reset() override;

      InputDomain getInputDomain() const override;

      std::string get_identifier() const override;
      std::string get_name() const override;
      std::string get_description() const override;
      std::string get_maker() const override;

      parameter_descriptors get_parameter_descriptors() const override;
      float get_parameter(std::string) const override;
      void set_parameter(std::string, float) override;

      programs get_programs() const override;
      std::string get_current_program() const override;
      void select_program(std::string) override;

      size_t getPreferredStepSize() const override;
      size_t getPreferredBlockSize() const override;

      size_t getMinChannelCount() const override;
      size_t getMaxChannelCount() const override;

      OutputList getOutputDescriptors() const override;

      FeatureSet process(const float *const *inputBuffers, real_time timestamp) override;

      FeatureSet getRemainingFeatures() override;

    protected:
      void convertFeatures(VampFeatureList *, FeatureSet &);

      const VampPluginDescriptor *m_descriptor;
      VampPluginHandle m_handle;
  };
}
