#pragma once

#include "vamp-hostsdk/PluginWrapper.h"

namespace vamp::host 
{
  class PluginInputDomainAdapter : public PluginWrapper
  {
    public:
      PluginInputDomainAdapter(plugin* p);
      virtual ~PluginInputDomainAdapter();
      
      bool initialise(size_t channels, size_t stepSize, size_t blockSize);
      void reset();

      InputDomain getInputDomain() const;

      size_t getPreferredStepSize() const;
      size_t getPreferredBlockSize() const;

      FeatureSet process(const float *const *inputBuffers, real_time timestamp);

      enum ProcessTimestampMethod {
          ShiftTimestamp,
          ShiftData,
          NoShift
      };

      void setProcessTimestampMethod(ProcessTimestampMethod);
      ProcessTimestampMethod getProcessTimestampMethod() const;
      real_time getTimestampAdjustment() const;

      enum WindowType {

          RectangularWindow    = 0,

          BartlettWindow       = 1, /// synonym for RectangularWindow
          TriangularWindow     = 1, /// synonym for BartlettWindow

          HammingWindow        = 2,

          HanningWindow        = 3, /// synonym for HannWindow
          HannWindow           = 3, /// synonym for HanningWindow

          BlackmanWindow       = 4,

          NuttallWindow        = 7,

          BlackmanHarrisWindow = 8
      };

      WindowType getWindowType() const;
      void setWindowType(WindowType type);

    protected:
      class Impl;
      Impl *m_impl;
  };
}
