#pragma once

#include "vamp-sdk/Plugin.h"

#include <vector>
#include <string>

namespace vamp::host
{

  class PluginLoader
  {
  public:
      static PluginLoader *getInstance();
      typedef std::string PluginKey;
      typedef std::vector<PluginKey> PluginKeyList;
      typedef std::vector<std::string> PluginCategoryHierarchy;
      PluginKeyList listPlugins();

      PluginKeyList listPluginsIn(std::vector<std::string> libraryNames);
      PluginKeyList listPluginsNotIn(std::vector<std::string> libraryNames);
      enum AdapterFlags {

          ADAPT_INPUT_DOMAIN  = 0x01,
          ADAPT_CHANNEL_COUNT = 0x02,
          ADAPT_BUFFER_SIZE   = 0x04,

          ADAPT_ALL_SAFE      = 0x03,

          ADAPT_ALL           = 0xff
      };

      plugin *loadPlugin(PluginKey key,
                         float inputSampleRate,
                         int adapterFlags = 0);
      
      PluginKey composePluginKey(std::string libraryName,
                                 std::string identifier);

      PluginCategoryHierarchy getPluginCategory(PluginKey plugin);
      std::string getLibraryPathForPlugin(PluginKey plugin);

    protected:
      PluginLoader();
      virtual ~PluginLoader();

      class Impl;
      Impl *m_impl;

      static PluginLoader *m_instance;
  };
}
