#pragma once

#include "vamp-hostsdk/PluginWrapper.h"

#include <set>

namespace vamp::host 
{
  class PluginSummarisingAdapter : public PluginWrapper
  {
    public:
      PluginSummarisingAdapter(plugin* p); 
      virtual ~PluginSummarisingAdapter();

      bool initialise(size_t channels, size_t stepSize, size_t blockSize);

      void reset();

      FeatureSet process(const float *const *inputBuffers, real_time timestamp);
      FeatureSet getRemainingFeatures();

      typedef std::set<real_time> SegmentBoundaries;

      void setSummarySegmentBoundaries(const SegmentBoundaries &);

      enum SummaryType {
        Minimum            = 0,
        Maximum            = 1,
        Mean               = 2,
        Median             = 3,
        Mode               = 4,
        Sum                = 5,
        Variance           = 6,
        StandardDeviation  = 7,
        Count              = 8,

        UnknownSummaryType = 999
      };

      enum AveragingMethod 
      {
        SampleAverage         = 0,
        ContinuousTimeAverage = 1
      };

      FeatureList getSummaryForOutput(int output, SummaryType type, AveragingMethod method = SampleAverage);
      FeatureSet getSummaryForAllOutputs(SummaryType type, AveragingMethod method = SampleAverage);

    protected:
      class Impl;
      Impl *m_impl;
  };
}
