#pragma once

#include "vamp-sdk/Plugin.h"

namespace vamp::host 
{
  class PluginWrapper : public plugin
  {
    public:
      virtual ~PluginWrapper();
      
      bool initialise(size_t channels, size_t stepSize, size_t blockSize) override;
      void reset() override;

      InputDomain getInputDomain() const override;

      std::string get_identifier() const override;
      std::string get_name() const override;
      std::string get_description() const override;
      std::string get_maker() const override;

      parameter_descriptors get_parameter_descriptors() const override;
      float get_parameter(std::string) const override;
      void set_parameter(std::string, float) override;

      programs get_programs() const override;
      std::string get_current_program() const override;
      void select_program(std::string) override;

      size_t getPreferredStepSize() const override;
      size_t getPreferredBlockSize() const override;

      size_t getMinChannelCount() const override;
      size_t getMaxChannelCount() const override;

      OutputList getOutputDescriptors() const override;

      FeatureSet process(const float *const *inputBuffers, real_time timestamp) override;

      FeatureSet getRemainingFeatures() override;

      template <typename WrapperType>
      WrapperType* getWrapper() 
      {
        WrapperType *w = dynamic_cast<WrapperType *>(this);
        if (w) return w;
        PluginWrapper *pw = dynamic_cast<PluginWrapper *>(m_plugin);
        if (pw) return pw->getWrapper<WrapperType>();
        return 0;
      }

    protected:
      PluginWrapper(plugin* p);
      plugin* m_plugin;
  };

}

