#pragma once

#include <iostream>
#include <string>

#define ONE_BILLION 1'000'000'000
struct timeval;

namespace vamp 
{
  struct real_time
  {
    public:
      real_time() = default;
      real_time(int s, int n);
      real_time(const real_time& r) : sec{ r.sec }, nsec{ r.nsec } { }

    public:
      int sec { 0 };
      int nsec { 0 };

    public:
      static real_time from_seconds(double sec);
      static real_time from_milliseconds(int msec);
      static real_time from_timeval(const struct timeval &);
      // real_time to frame
      static long rt2f(const real_time &r, unsigned int sample_rate);
      // frame to real_time
      static real_time f2rt(long frame, unsigned int sample_rate);
      static const real_time zero_time;

    public:
      int usec() const { return nsec / 1000; }
      int msec() const { return nsec / 1000000; }

      std::string to_string() const;
      std::string to_text(bool fixed_dp = false) const;

    public:
      real_time &operator=(const real_time &r) 
      {
        sec = r.sec; nsec = r.nsec; return *this;
      }

      real_time operator+(const real_time &r) const 
      {
        return real_time(sec + r.sec, nsec + r.nsec);
      }

      real_time operator-(const real_time &r) const 
      {
        return real_time(sec - r.sec, nsec - r.nsec);
      }

      real_time operator-() const 
      {
        return real_time(-sec, -nsec);
      }

      bool operator <(const real_time &r) const 
      {
        if (sec == r.sec) return nsec < r.nsec;
        else return sec < r.sec;
      }

      bool operator >(const real_time &r) const 
      {
        if (sec == r.sec) return nsec > r.nsec;
        else return sec > r.sec;
      }

      bool operator==(const real_time &r) const 
      {
        return (sec == r.sec && nsec == r.nsec);
      }
   
      bool operator!=(const real_time &r) const 
      {
        return !(r == *this);
      }
   
      bool operator>=(const real_time &r) const 
      {
        if (sec == r.sec) return nsec >= r.nsec;
        else return sec >= r.sec;
      }

      bool operator<=(const real_time &r) const 
      {
        if (sec == r.sec) return nsec <= r.nsec;
        else return sec <= r.sec;
      }

      real_time operator/(int d) const;
      double operator/(const real_time &r) const;

      friend std::ostream &operator<<(std::ostream &out, const real_time &rt);
  };
}
